# -*- coding: utf-8 -*-
"""TODO: DOCSTRING."""

import pyglet
import graphx


class GameWindow(pyglet.window.Window):
    """Manager for the pyglet game window."""

    def __init__(self, width, height, **kargs):
        """Init GameWindow as Pyglet Window."""
        super(GameWindow, self).__init__(width, height, **kargs)

        self.graphx_manager = None

        self.batch_terrain = pyglet.graphics.Batch()   # Tiles
        self.batch_entities = pyglet.graphics.Batch()  # NPCs
        self.batch_player = pyglet.graphics.Batch()    # Player
        self.batch_effects = pyglet.graphics.Batch()   # Effects
        self.batch_overlay = pyglet.graphics.Batch()   # UI

    def load_tileset(self, graphx_manager):
        """Load tileset from file, with given size."""
        if not isinstance(graphx_manager, graphx.GraphX):
            raise TypeError('''Argument must be of type GraphX, found {}\
                            '''.format(type(graphx_manager)))
        self.graphx_manager = graphx_manager

    def on_draw(self):
        """Call the pyglet window draws on each batch."""
        self.clear()

        # Order matters!
        self.batch_terrain.draw()  # Layer: Bottom
        self.batch_entities.draw()
        self.batch_player.draw()
        self.batch_effects.draw()
        self.batch_overlay.draw()  # Layer: Top



"""TODO DOCSTRING."""


class Entity(object):
    """A class representing a NPC or any type of in-world object."""

    def __init__(self, name, x=-1, y=-1):
        """Construct a primitive entity."""
        self.name = name
        self.x = x
        self.y = y



# -*- coding: utf-8 -*-
"""TODO: DOCSTRING."""

import pyglet


class GraphX(object):
    """A graphics manager for the game, this includes colors, tiles, etc..."""

    def __init__(self):
        """Init graphx manager."""
        pass



"""TODO DOCSTRING."""

import tiles


class Map(object):
    """A class representation of a map logic."""

    def __init__(self, size_x, size_y, size_z):
        """Construct a map of given sizes."""
        if not isinstance(size_x, int) or\
           not isinstance(size_y, int) or\
           not isinstance(size_z, int):
            raise TypeError('The map dimensions must be positive integers.')

        if size_x <= 0 or size_y <= 0 or size_z <= 0:
            raise AttributeError('The map dimensions must be POSITIVE ints.')

        self.dimensions = (size_x, size_y, size_z)
        self.tiles = []


class MapGenerators(object):
    """A set of methods for generating maps."""

    def flat_map(self, size_x, size_y, mode=0):
        """Generate a flat map."""
        tile = tiles.Tile(0, 0, 0)
        return [[tile for x in range(size_x)] for y in range(size_y)]



"""TODO DOCSTRING."""


class Tile(object):
    """A class representation of a tile logic."""

    def __init__(self, x, y, z):
        """Construct a tile."""
        if not isinstance(x, int) or\
           not isinstance(y, int) or\
           not isinstance(z, int):
            raise TypeError('The tile position must be a natural number.')

        if x < 0 or y < 0 or z < 0:
            raise AttributeError('The tile positions must be natural numbers.')

        self.position = (x, y, z)

    def __str__(self):
        """Return a string representation of Tile."""
        return 'Tile(Position{})'.format(self.position)

    def __repr__(self):
        """Return the __str__ method output."""
        return self.__str__()
